<?php
/* vim: set expandtab tabstop=4 shiftwidth=4: */
/**
 * PHP versions 4 and 5
 *
 * LICENSE: Copyright 2007 Masashi Shinbara. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * o Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.
 * o Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE FREEBSD PROJECT "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE FREEBSD PROJECT OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation are
 * those of the authors and should not be interpreted as representing official
 * policies, either expressed or implied, of The PEAR Group.
 *
 * @category  Web Services
 * @package   Services_Rakuten
 * @author    Masashi Shinbara <shin1x1@gmail.com>
 * @copyright 2007 Masashi Shinbara
 * @license   http://www.freebsd.org/copyright/freebsd-license.html 2 Clause BSD License
 * @filesource
 * @see       http://webservice.rakuten.co.jp/
 */
 
require_once 'PEAR.php';
require_once 'HTTP/Request.php';
require_once 'XML/Unserializer.php';

/**
 * Class for accessing and retrieving information from Rakuten's Web API.
 *
 * @package Services_Rankuten
 * @author  Masashi Shinbara <shin1x1@gmail.com>
 * @access  public
 * @version Release: 0.1.0
 * @uses    PEAR
 * @uses    HTTP_Request
 * @uses    XML_Unserializer
 */
class Services_Rakuten
{
    /**
     * @var string
     */
    var $developerId = null;
    /**
     * @var string
     */
    var $affiliateId = null;
    /**
     * @var string
     */
    var $baseUrl = null;
    /**
     * @var string
     */
    var $lastUrl = null;

    /**
     * @var string
     */
    var $status = null;
    /**
     * @var string
     */
    var $statusMessage = null;
    /**
     * @var array
     */
    var $data = null;

    /**
     * Constructor
     *
     * @access public
     * @param  string $developerId
     * @param  string $affiliateId
     */
    function Services_Rakuten($developerId = null, $affiliateId = null
                            , $baseUrl = 'http://api.rakuten.co.jp/rws/1.1/rest') {
        if (!is_null($developerId)) {
            $this->developerId = $developerId;
        }
        if (!is_null($affiliateId)) {
            $this->affiliateId = $affiliateId;
        }

        $this->baseUrl = $baseUrl;
    }

    /**
     * Send ItemSearch operation
     *
     * @param  string  $keyword
     * @param  integer $genreId
     * @param  array   $options
     * @see http://webservice.rakuten.co.jp/api/itemsearch/
     */
    function doItemSearch($keyword = null, $genreId = null, $options = array()) {
        if (is_null($keyword) && is_null($genreId)) {
            return PEAR::raiseError('[ItemSearch] keyword or genreId have not been set.');
        }

        if (!empty($keyword)) {
          $options['keyword'] = $keyword;
        }
        if (!empty($genreId)) {
          $options['genreId'] = $genreId;
        }

        $this->sendRequest('ItemSearch', $options);
    }

    /**
     * Send GenreSearch operation
     *
     * @param  integer $genreId
     * @param  array   $options
     * @see http://webservice.rakuten.co.jp/api/genresearch/
     */
    function doGenreSearch($genreId, $options = array()) {
        $options['genreId'] = $genreId;
        $this->sendRequest('GenreSearch', $options);
    }

    /**
     * Send ItemCodeSearch operation
     *
     * @param  string  $itemCode
     * @param  array   $options
     * @see http://webservice.rakuten.co.jp/api/itemcodesearch/
     */
    function doItemCodeSearch($itemCode, $options = array()) {
        $options['itemCode'] = $itemCode;
        $this->sendRequest('ItemCodeSearch', $options);
    }

    /**
     * Send BookSearch operation
     *
     * @param  integer $keyword
     * @param  array   $options
     * @see http://webservice.rakuten.co.jp/api/booksearch/
     */
    function doBookSearch($keyword = null, $genreId = null, $options = array()) {
        if (is_null($keyword) && is_null($genreId)) {
            return PEAR::raiseError('[BookSearch] keyword or genreId have not been set.');
        }

        if (!empty($keyword)) {
          $options['keyword'] = $keyword;
        }
        if (!empty($genreId)) {
          $options['genreId'] = $genreId;
        }

        $this->sendRequest('BookSearch', $options);
    }

    /**
     * Sends the request to Rakuten Web API.
     *
     * @param  string  $operation
     * @param  array   $options
     */
    function sendRequest($operation, $options = array()) {
        if (is_null($this->developerId)) {
            return PEAR::raiseError('Developers Id have not been set.');
        }

        $params = $options;
        $params['developerId'] = $this->developerId;
        $params['affiliateId'] = $this->affiliateId;
        $params['version']     = $this->getApiVersion($operation);
        $params['operation']   = $operation;

        $query = "";
        foreach ($params as $key => $value) {
          if (!empty($query)) {
            $query .= "&";
          }
          $query .= sprintf("%s=%s", $key, urlencode($value));
        }

        $this->data = null;
        $this->status = null;
        $this->statusMessage = null;

        $this->lastUrl = $this->baseUrl . '?' . $query;

        $http =& new HTTP_Request($this->lastUrl);
        $http->addHeader('User-Agent', 'Services_Rankuten');
        $http->sendRequest();

        if ($http->getResponseCode() != 200) {
            return PEAR::raiseError('Rakuten return HTTP ' . $http->getResponseCode());
        }
        $result = $http->getResponseBody();

        $xml =& new XML_Unserializer();
        $xml->unserialize($result);
        $data = $xml->getUnserializedData();

        $this->status = $data['Header']['Status'];
        $this->statusMessage = $data['Header']['StatusMsg'];

        if ($this->status == 'Success') {
          $this->data = $data['Body'][$operation];
        }
    }

    /**
     * get result body data
     *
     * @return array
     */
    function &getResultData() {
        return $this->data;
    }

    /**
     * get last url
     *
     * @return string
     */
    function getLastUrl() {
        return $this->lastUrl;
    }

    /**
     * get Rakuten API version
     *
     * @param  stinrg $operation
     * @return string
     */
    function getApiVersion($operation) {
        $versions = array('ItemSearch'  => '2007-01-17'
                        , 'GenreSearch' => '2006-12-26'
                        , 'ItemCodeSearch' => '2007-01-17'
                        , 'BookSearch' => '2007-01-17'
                        );

        return $versions[$operation];
    }
}
?>
